#ifndef lint
static char const 
yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.28 2000/01/17 02:04:06 bde Exp $";
#endif
#include <stdlib.h>
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYLEX yylex()
#define YYEMPTY -1
#define yyclearin (yychar=(YYEMPTY))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING() (yyerrflag!=0)
static int yygrowstack();
#define YYPREFIX "yy"
#line 2 "aicasm_gram.y"
/*
 * Parser for the Aic7xxx SCSI Host adapter sequencer assembler.
 *
 * Copyright (c) 1997, 1998, 2000 Justin T. Gibbs.
 * Copyright (c) 2001, 2002 Adaptec Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification.
 * 2. Redistributions in binary form must reproduce at minimum a disclaimer
 *    substantially similar to the "NO WARRANTY" disclaimer below
 *    ("Disclaimer") and any redistribution must be conditioned upon
 *    including a substantially similar Disclaimer requirement for further
 *    binary redistribution.
 * 3. Neither the names of the above-listed copyright holders nor the names
 *    of any contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * NO WARRANTY
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * $Id: //depot/aic7xxx/aic7xxx/aicasm/aicasm_gram.y#21 $
 *
 * $FreeBSD$
 */

#include <inttypes.h>
#include <sys/types.h>

#include <inttypes.h>
#include <regex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>

#ifdef __linux__
#include "../queue.h"
#else
#include <sys/queue.h>
#endif

#include "aicasm.h"
#include "aicasm_symbol.h"
#include "aicasm_insformat.h"

int yylineno;
char *yyfilename;
char stock_prefix[] = "aic_";
char *prefix = stock_prefix;
char *patch_arg_list;
char *versions;
static char errbuf[255];
static char regex_pattern[255];
static symbol_t *cur_symbol;
static symbol_t *field_symbol;
static symbol_t *scb_or_sram_symbol;
static symtype cur_symtype;
static symbol_ref_t accumulator;
static symbol_ref_t mode_ptr;
static symbol_ref_t allones;
static symbol_ref_t allzeros;
static symbol_ref_t none;
static symbol_ref_t sindex;
static int instruction_ptr;
static int num_srams;
static int sram_or_scb_offset;
static int download_constant_count;
static int in_critical_section;
static u_int enum_increment;
static u_int enum_next_value;

static void process_field(int field_type, symbol_t *sym, int mask);
static void initialize_symbol(symbol_t *symbol);
static void add_macro_arg(const char *argtext, int position);
static void add_macro_body(const char *bodytext);
static void process_register(symbol_t **p_symbol);
static void format_1_instr(int opcode, symbol_ref_t *dest,
			   expression_t *immed, symbol_ref_t *src, int ret);
static void format_2_instr(int opcode, symbol_ref_t *dest,
			   expression_t *places, symbol_ref_t *src, int ret);
static void format_3_instr(int opcode, symbol_ref_t *src,
			   expression_t *immed, symbol_ref_t *address);
static void test_readable_symbol(symbol_t *symbol);
static void test_writable_symbol(symbol_t *symbol);
static void type_check(symbol_t *symbol, expression_t *expression, int and_op);
static void make_expression(expression_t *immed, int value);
static void add_conditional(symbol_t *symbol);
static void add_version(const char *verstring);
static int  is_download_const(expression_t *immed);

#define SRAM_SYMNAME "SRAM_BASE"
#define SCB_SYMNAME "SCB_BASE"
#line 115 "aicasm_gram.y"
typedef union {
	u_int		value;
	char		*str;
	symbol_t	*sym;
	symbol_ref_t	sym_ref;
	expression_t	expression;
} YYSTYPE;
#line 137 "aicasm_gram.tab.c"
#define YYERRCODE 256
#define T_REGISTER 257
#define T_CONST 258
#define T_EXPORT 259
#define T_DOWNLOAD 260
#define T_SCB 261
#define T_SRAM 262
#define T_ALIAS 263
#define T_SIZE 264
#define T_EXPR_LSHIFT 265
#define T_EXPR_RSHIFT 266
#define T_ADDRESS 267
#define T_ACCESS_MODE 268
#define T_MODES 269
#define T_DEFINE 270
#define T_SET_SRC_MODE 271
#define T_SET_DST_MODE 272
#define T_MODE 273
#define T_BEGIN_CS 274
#define T_END_CS 275
#define T_FIELD 276
#define T_ENUM 277
#define T_MASK 278
#define T_NUMBER 279
#define T_PATH 280
#define T_STRING 281
#define T_ARG 282
#define T_MACROBODY 283
#define T_CEXPR 284
#define T_EOF 285
#define T_INCLUDE 286
#define T_VERSION 287
#define T_PREFIX 288
#define T_PATCH_ARG_LIST 289
#define T_SHR 290
#define T_SHL 291
#define T_ROR 292
#define T_ROL 293
#define T_MVI 294
#define T_MOV 295
#define T_CLR 296
#define T_BMOV 297
#define T_JMP 298
#define T_JC 299
#define T_JNC 300
#define T_JE 301
#define T_JNE 302
#define T_JNZ 303
#define T_JZ 304
#define T_CALL 305
#define T_ADD 306
#define T_ADC 307
#define T_INC 308
#define T_DEC 309
#define T_STC 310
#define T_CLC 311
#define T_CMP 312
#define T_NOT 313
#define T_XOR 314
#define T_TEST 315
#define T_AND 316
#define T_OR 317
#define T_RET 318
#define T_NOP 319
#define T_ACCUM 320
#define T_ALLONES 321
#define T_ALLZEROS 322
#define T_NONE 323
#define T_SINDEX 324
#define T_MODE_PTR 325
#define T_A 326
#define T_SYMBOL 327
#define T_NL 328
#define T_IF 329
#define T_ELSE 330
#define T_ELSE_IF 331
#define T_ENDIF 332
#define UMINUS 333
const short yylhs[] = {                                        -1,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   20,   20,   21,   22,   23,   37,   24,   39,
   36,   38,   38,   40,   40,   40,   40,   40,   40,   40,
   40,   40,   40,   40,   40,   40,   40,   41,   42,   43,
   44,   18,   18,   17,   17,   55,   45,   57,   45,   45,
   58,   46,   59,   46,   56,   56,   60,   60,   47,   48,
   49,   50,   51,   52,   53,   54,    6,    6,    6,    6,
    6,    6,    6,    6,    6,    6,    6,    6,    6,   25,
   25,   61,   26,   26,   19,   19,   19,   16,   16,   62,
   63,   65,   27,   66,   67,   68,   28,   64,   64,   64,
   64,   69,   69,    1,    1,    1,    1,    3,    7,    8,
    8,    4,    5,    5,   10,   10,   30,   31,   32,   33,
    9,    9,   29,    2,    2,    2,    2,    2,    2,   34,
   34,   34,   34,   11,   11,   11,   11,   35,   35,   35,
   35,   35,   35,   35,   35,   35,   35,   35,   35,   35,
   35,   35,   12,   12,   12,   12,   35,   13,   13,   13,
   13,   14,   14,   15,   15,   35,   35,   35,   35,   35,
   35,
};
const short yylen[] = {                                         2,
    1,    2,    1,    2,    1,    2,    1,    2,    1,    2,
    1,    2,    1,    2,    1,    2,    1,    2,    1,    2,
    1,    2,    1,    2,    1,    2,    1,    2,    1,    2,
    1,    2,    4,    4,    3,    3,    3,    0,    3,    0,
    5,    1,    2,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    2,    2,    2,
    2,    1,    3,    1,    1,    0,    5,    0,    7,    3,
    0,    5,    0,    7,    1,    3,    1,    2,    3,    2,
    1,    1,    1,    1,    1,    1,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    2,    2,    1,    1,    3,
    3,    2,    2,    5,    0,    1,    3,    1,    2,    0,
    0,    0,    9,    0,    0,    0,    9,    0,    1,    1,
    2,    1,    2,    1,    4,    4,    1,    1,    1,    1,
    1,    1,    0,    2,    0,    1,    3,    3,    2,    2,
    0,    1,    3,    1,    3,    3,    1,    3,    3,    3,
    4,    2,    1,    1,    1,    1,    1,    7,    7,    5,
    5,    3,    7,    3,    4,    8,    6,    6,    5,    4,
    3,    2,    1,    1,    1,    1,    7,    1,    1,    1,
    1,    1,    1,    1,    1,    3,    7,    7,    7,    5,
    5,
};
const short yydefred[] = {                                      0,
   38,    0,  142,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  174,  173,  176,  175,    0,    0,
    0,    0,  178,  179,  180,  181,  156,  157,    0,    0,
    0,    0,    0,    0,  155,    0,  154,    0,    0,    0,
    0,    0,  153,    0,    0,    0,    0,    0,    1,    3,
    5,    7,    9,   11,   13,   15,   17,   19,   21,   23,
   25,   27,   29,   31,    0,    0,    0,  114,  110,  102,
    0,    0,  139,  140,    0,    0,    0,    0,    0,   98,
  127,    0,    0,    0,    0,  128,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  136,    0,    0,
    0,    0,  132,    0,    0,    0,    0,  172,    0,    0,
    0,  152,    2,    4,    6,    8,   10,   12,   14,   16,
   18,   20,   22,   24,   26,   28,   30,   32,    0,    0,
    0,    0,    0,    0,  103,    0,    0,   39,  101,  108,
    0,  100,    0,    0,  137,  138,    0,    0,   37,   35,
   36,    0,   99,   97,   96,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  164,    0,  162,    0,    0,    0,
    0,  171,  150,    0,  143,    0,    0,    0,    0,    0,
    0,  186,  106,    0,   40,  109,    0,  115,  111,   33,
   34,    0,    0,   95,  131,    0,    0,    0,    0,    0,
    0,    0,    0,   91,   92,    0,    0,    0,  170,    0,
  134,    0,    0,  165,    0,    0,    0,    0,    0,    0,
    0,  151,    0,    0,  145,  146,  148,  149,    0,    0,
    0,   58,    0,    0,  126,  125,    0,  191,    0,  190,
    0,  160,  161,    0,  184,  185,    0,  169,  183,  182,
    0,    0,    0,    0,    0,  107,  104,    0,    0,    0,
    0,    0,    0,    0,   81,   83,   84,   85,   86,   82,
    0,   42,   44,   45,   46,   47,   48,   49,   50,   51,
   52,   53,   54,   55,   56,   57,  116,  112,  168,  167,
    0,    0,    0,    0,    0,    0,    0,    0,   80,   59,
   60,   64,   65,   62,    0,    0,    0,    0,    0,    0,
   41,   43,    0,    0,    0,  163,  189,  188,  187,  159,
  158,  177,    0,    0,    0,    0,    0,    0,  122,    0,
    0,    0,    0,  166,   63,    0,    0,    0,   75,    0,
    0,    0,  117,  123,  113,    0,    0,   67,    0,    0,
   72,    0,   76,    0,   69,   74,
};
const short yydgoto[] = {                                      44,
   86,  134,   87,   93,  172,  206,   89,  207,   45,  100,
   46,   47,   48,  261,  257,  142,  314,  315,  194,   49,
   50,   51,   52,   53,   54,   55,   56,   57,   58,   59,
   60,   61,   62,   63,   64,  339,   66,  281,  241,  282,
  283,  284,  285,  286,  287,  288,  289,  290,  291,  292,
  293,  294,  295,  296,  317,  348,  346,  319,  350,  349,
   65,  144,  244,  341,  324,  143,  243,  323,  342,
};
const short yysindex[] = {                                    476,
    0, -294,    0,  -72,  -63, -248, -175, -148,   49,   78,
  -16,   88,   93,   96,    0,    0,    0,    0,  -14, -230,
 -230, -230,    0,    0,    0,    0,    0,    0, -230, -230,
 -255, -264, -230, -230,    0, -230,    0, -230,  113, -134,
  -96, -120,    0,  476, -136, -230, -230,  -44,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  -34, -135,   55,    0,    0,    0,
  135,  138,    0,    0,  -81,  -80,  -73,  -71,  -70,    0,
    0,  110,  -17,  -17,  -17,    0,  162,  178, -180,  110,
    0,  168, -180, -134,  169,  170,  170,    0, -134,  158,
 -230,  159,    0,  175,  170,  180,  182,    0,  163,  104,
  -56,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  171,  186,
  187,   25,   87,  192,    0,  -29,  129,    0,    0,    0,
  -25,    0,  -10,  -10,    0,    0,  193,  226,    0,    0,
    0, -245,    0,    0,    0,  132,   -3,  -17,  -17,  -17,
  -17,  -17,  -17,  -17,  -17,  -44, -230,  -44,  222, -230,
 -230, -134, -134,  227,    0,  245,    0,   -3, -134,   -3,
   -3,    0,    0,  173,    0,   -3,  -17,   21,   35,   47,
   48,    0,    0,    9,    0,    0,   50,    0,    0,    0,
    0,  218,  232,    0,    0,  178, -134,  124,  124,  140,
   -7,   60,   60,    0,    0,  274, -134,  276,    0,  305,
    0,  292,  296,    0,   -3, -185,  309, -182,  178, -180,
  170,    0,  170,  160,    0,    0,    0,    0,   89,   90,
  195,    0,  106,  106,    0,    0,  315,    0,  317,    0,
  -17,    0,    0, -134,    0,    0,  -44,    0,    0,    0,
  -44,  -44, -134, -134, -134,    0,    0,   51,   98,  115,
 -238,   52,   65,   72,    0,    0,    0,    0,    0,    0,
  551,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 -134,  341,  342,  343,  344,  345,  348,  349,    0,    0,
    0,    0,    0,    0,  365,  -17,  288,  -17,  290,  -17,
    0,    0, -252, -252,  351,    0,    0,    0,    0,    0,
    0,    0, -238,  178,   97,  178,   97,  178,    0, -135,
  289, -135,  298,    0,    0,  304,  -17,  -39,    0,  307,
  -24, -135,    0,    0,    0,   97,  178,    0,   97,   97,
    0,  -12,    0,   -1,    0,    0,
};
const short yyrindex[] = {                                    101,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  372,  372,    0,    0,    0,    0,    0,    0,    0,  372,
    0,    0,    0,  101,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   91,    0,    0,    0,    0,    0,  -55,    0,  -20,
   54,    0,    0,  372,    0,  -52,  -52,    0,  372,    0,
    0,    0,    0,    0,  -52,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  373,  374,    0,    0,   46,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  372,  372,    0,    0,    0,    0,    0,  372,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   18,  372,  117,  223,  406,
  328,  -30,   40,    0,    0,    0,  372,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   32,    0,
  -52,    0,  -52,  -52,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  372,    0,    0,    0,    0,    0,    0,
    0,    0,  372,  372,  372,    0,    0,    0,    0,    0,
    0,  313,  322,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  372,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  287,    0,    0,    0,    0,    0,
    0,    0,  321,  321,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  533,    0,  325,    0,  567,    0,  324,
    0,  329,    0,    0,    0,    0,    1,    0,    0,    0,
    0,  330,    0,    0,    0,    0,    2,    0,    0,    0,
    0,    0,    0,    0,    0,    0,
};
const short yygindex[] = {                                      0,
  -11, -147,   27,   19,  -38,  -19, -171,  -75,    0,  335,
    0,    0,  -77,    0,    0,    0,  123,    0,    0,  413,
  416,  417,  422,  423,  424,  425,  426,  430,  431,  432,
  433,  434,  435,  436,  437,  -65,    0,    0,    0,  201,
    4,  -92,    0, -143,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, -268,    0,    0,    0,  125,
    0,    0,    0,  161,    0,    0,    0,    0,  143,
};
#define YYTABLESIZE 892
const short yytable[] = {                                      88,
  138,  133,  112,  129,  359,  136,  133,   89,   91,  230,
   89,  166,   89,   89,   89,  168,  271,   76,  216,  359,
  218,  103,   85,  124,  103,   85,  107,   83,   89,  101,
   83,  359,   67,  202,  164,  162,   85,  163,  124,  165,
  312,   83,  359,   75,   77,   78,   92,   94,   95,  240,
   68,  104,  239,   98,  106,   96,   97,   99,  173,   69,
  105,  130,   98,  154,  155,  156,  179,  188,  351,  189,
   81,   90,  130,  131,  137,  130,  130,   90,   70,  301,
   90,  203,   90,   90,   90,  358,  105,  362,  313,  105,
  130,  364,   89,   89,   89,   81,   90,  128,   90,  141,
  361,  164,  226,   71,  228,  231,  165,   73,   84,  303,
  233,   84,  365,  304,  305,  255,  256,   23,   24,   25,
  259,  260,   84,  366,   26,   77,   78,  176,   99,  190,
   72,  191,   99,   99,  124,   99,   74,   99,  208,  209,
  210,  211,  212,  213,  214,  215,  198,  199,   77,  254,
  297,  298,  262,   78,   93,  103,   79,   93,  103,  103,
   93,  229,   90,   90,   90,  164,  162,  234,  163,  161,
  165,  108,  204,  164,  162,   93,  163,  161,  165,  340,
  340,  164,  162,   98,  163,  217,  165,  110,  220,  221,
  129,  137,  263,  145,  264,  265,  146,  161,  147,  148,
  152,  164,  162,  171,  163,  157,  165,  149,  111,  150,
  151,  167,  170,  171,   99,  161,  175,  177,  178,  164,
  162,  182,  163,  180,  165,  181,  183,  184,  185,  186,
  187,   88,   89,   89,   89,   89,   89,   89,   89,   93,
   93,   93,  129,  129,  129,   89,   89,   89,  135,  129,
  192,  195,  193,  196,  200,  160,  197,  158,  159,  201,
   94,   80,  129,   94,   80,  133,   94,   89,   89,   89,
   89,   89,   89,   89,   89,   80,  354,  124,  124,  124,
  219,   94,  132,  160,  124,  224,  354,   89,  225,   89,
   89,   89,   89,   89,   89,  232,  334,  124,  336,  235,
  338,  160,   90,   90,   90,   90,   90,   90,   90,  153,
  245,   81,   82,  236,  139,   90,   90,   90,  130,  130,
  130,  130,  205,  153,  246,  237,  238,  357,  242,  129,
  129,  129,  248,  140,  250,  130,  129,   90,   90,   90,
   90,   90,   90,   90,   90,   94,   94,   94,  251,  130,
  252,  132,  132,  132,  253,   99,   99,   90,  132,   90,
   90,   90,   90,   90,   90,   88,  102,  258,   88,  269,
  266,   88,  267,  299,  109,  300,  310,  309,  316,   93,
   93,   93,   93,   93,   93,   93,   88,  311,   99,   99,
   99,  318,   93,   93,   93,   99,  158,  159,  320,  326,
  327,  328,  329,  330,  158,  159,  331,  332,  333,  344,
  335,   61,  337,  353,   93,   93,   93,   93,   93,   93,
   93,   93,  355,  347,  158,  159,  356,  141,  169,  360,
  135,  144,  147,  174,   93,   66,   93,   93,   93,   93,
   93,   93,  158,  159,   71,  118,   87,   73,  119,   87,
   88,   88,   88,  120,  121,  345,  113,  268,  269,  114,
  115,  197,  270,  271,   87,  116,  117,  118,  119,  120,
  272,  273,  274,  121,  122,  123,  124,  125,  126,  127,
  128,  322,  352,  363,  343,   94,   94,   94,   94,   94,
   94,   94,    0,    0,    0,    0,    0,    0,   94,   94,
   94,    0,    0,    0,    0,    0,  222,  223,    0,    0,
    0,    0,    0,  227,  275,  276,  277,  278,  279,  280,
   94,   94,   94,   94,   94,   94,   94,   94,   87,   87,
   87,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   94,  247,   94,   94,   94,   94,   94,   94,    0,   61,
   61,  249,    0,   61,   61,   61,    0,    0,    0,    0,
    0,    0,   61,   61,   61,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  302,    0,
   88,   88,    0,    0,   88,   88,   88,  306,  307,  308,
   43,    0,    0,   88,   88,   88,   61,   61,   61,   61,
   61,   61,    0,   61,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   88,   88,   88,   88,   88,
   88,   88,   88,    0,    0,  325,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   88,    0,   88,   88,   88,
   88,   88,   88,    0,    0,   68,    0,   70,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   87,   87,
    0,    0,   87,   87,   87,  321,    0,    0,    0,    0,
    0,   87,   87,   87,    0,    0,    0,    0,    0,    0,
    0,   79,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   87,   87,   87,   87,   87,   87,   87,
   87,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   87,    0,   87,   87,   87,   87,   87,
   87,    0,    1,    2,    3,    0,    4,    5,    0,    0,
    0,    0,    0,    0,    0,    6,    7,    8,    0,    9,
   10,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   11,   12,   13,   14,   15,   16,   17,   18,   19,
   20,   21,   22,   23,   24,   25,    0,    0,    0,    0,
   26,   27,   28,   29,   30,   31,   32,   33,   34,   35,
   36,   37,   38,   39,   40,   70,   70,    0,    0,   70,
   70,   70,    0,    0,   41,   42,    0,    0,   70,   70,
   70,    0,    0,  268,  269,    0,    0,  197,  270,  271,
    0,    0,    0,    0,    0,    0,  272,  273,  274,   79,
   79,    0,    0,   79,   79,   79,    0,    0,    0,    0,
    0,    0,   79,   79,   79,    0,    0,    0,    0,    0,
    0,    0,   70,   70,   70,   70,   70,   70,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  275,  276,  277,  278,  279,  280,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   79,   79,   79,   79,
   79,   79,
};
const short yycheck[] = {                                      19,
   66,   46,  123,   59,   44,   40,   59,   38,   20,  181,
   41,   89,   43,   44,   45,   93,  269,   34,  166,   44,
  168,   33,   40,   44,   36,   40,   38,   45,   59,  294,
   45,   44,  327,  279,   42,   43,   40,   45,   59,   47,
  279,   45,   44,   60,   44,   44,   20,   21,   22,   41,
  123,   33,   44,  318,   36,   29,   30,   31,   97,  123,
   34,   44,  318,   83,   84,   85,  105,   43,  337,   45,
  326,  327,   46,   47,  327,   44,   59,   38,  327,  251,
   41,  327,   43,   44,   45,  125,   41,  356,  327,   44,
   59,  360,  123,  124,  125,  326,  327,   44,   59,   45,
  125,   42,  178,  279,  180,  181,   47,   59,  126,  257,
  186,  126,  125,  261,  262,  301,  302,  298,  299,  300,
  303,  304,  126,  125,  305,  125,  125,  101,   38,   43,
  279,   45,   42,   43,   44,   45,   59,   47,  158,  159,
  160,  161,  162,  163,  164,  165,  143,  144,   61,  225,
  243,  244,  230,   61,   38,  167,   61,   41,  170,  171,
   44,  181,  123,  124,  125,   42,   43,  187,   45,   38,
   47,   59,   41,   42,   43,   59,   45,   38,   47,  323,
  324,   42,   43,  318,   45,  167,   47,  284,  170,  171,
  327,  327,  231,   59,  233,  234,   59,   38,  280,  280,
   91,   42,   43,   44,   45,   44,   47,  281,  329,  281,
  281,   44,   44,   44,  124,   38,   59,   59,   44,   42,
   43,   59,   45,   44,   47,   44,  123,  284,   58,   44,
   44,  251,  263,  264,  265,  266,  267,  268,  269,  123,
  124,  125,  298,  299,  300,  276,  277,  278,  283,  305,
   59,  123,  282,  279,   62,  124,  267,  265,  266,   34,
   38,  279,  318,   41,  279,  318,   44,  298,  299,  300,
  301,  302,  303,  304,  305,  279,  342,  298,  299,  300,
   59,   59,  327,  124,  305,   59,  352,  318,   44,  320,
  321,  322,  323,  324,  325,  123,  316,  318,  318,  279,
  320,  124,  263,  264,  265,  266,  267,  268,  269,  327,
   93,  326,  327,  279,  260,  276,  277,  278,  301,  302,
  303,  304,  326,  327,   93,  279,  279,  347,  279,  298,
  299,  300,   59,  279,   59,  318,  305,  298,  299,  300,
  301,  302,  303,  304,  305,  123,  124,  125,   44,  318,
   59,  298,  299,  300,   59,  265,  266,  318,  305,  320,
  321,  322,  323,  324,  325,   38,   32,   59,   41,  264,
  282,   44,  283,   59,   40,   59,  279,  327,  327,  263,
  264,  265,  266,  267,  268,  269,   59,  273,  298,  299,
  300,  327,  276,  277,  278,  305,  265,  266,  327,   59,
   59,   59,   59,   59,  265,  266,   59,   59,   44,   59,
  123,  125,  123,  125,  298,  299,  300,  301,  302,  303,
  304,  305,  125,  327,  265,  266,  123,  327,   94,  123,
   59,   59,   59,   99,  318,  123,  320,  321,  322,  323,
  324,  325,  265,  266,  123,  125,   41,  123,  125,   44,
  123,  124,  125,  125,  125,  333,   44,  263,  264,   44,
   44,  267,  268,  269,   59,   44,   44,   44,   44,   44,
  276,  277,  278,   44,   44,   44,   44,   44,   44,   44,
   44,  281,  340,  359,  324,  263,  264,  265,  266,  267,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,  276,  277,
  278,   -1,   -1,   -1,   -1,   -1,  172,  173,   -1,   -1,
   -1,   -1,   -1,  179,  320,  321,  322,  323,  324,  325,
  298,  299,  300,  301,  302,  303,  304,  305,  123,  124,
  125,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,  207,  320,  321,  322,  323,  324,  325,   -1,  263,
  264,  217,   -1,  267,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,  276,  277,  278,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  254,   -1,
  263,  264,   -1,   -1,  267,  268,  269,  263,  264,  265,
  125,   -1,   -1,  276,  277,  278,  320,  321,  322,  323,
  324,  325,   -1,  327,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  298,  299,  300,  301,  302,
  303,  304,  305,   -1,   -1,  301,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,  320,  321,  322,
  323,  324,  325,   -1,   -1,  123,   -1,  125,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  263,  264,
   -1,   -1,  267,  268,  269,  125,   -1,   -1,   -1,   -1,
   -1,  276,  277,  278,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  125,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  298,  299,  300,  301,  302,  303,  304,
  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  318,   -1,  320,  321,  322,  323,  324,
  325,   -1,  257,  258,  259,   -1,  261,  262,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  270,  271,  272,   -1,  274,
  275,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  286,  287,  288,  289,  290,  291,  292,  293,  294,
  295,  296,  297,  298,  299,  300,   -1,   -1,   -1,   -1,
  305,  306,  307,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,  319,  263,  264,   -1,   -1,  267,
  268,  269,   -1,   -1,  329,  330,   -1,   -1,  276,  277,
  278,   -1,   -1,  263,  264,   -1,   -1,  267,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,  276,  277,  278,  263,
  264,   -1,   -1,  267,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,  276,  277,  278,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  320,  321,  322,  323,  324,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  320,  321,  322,  323,  324,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  320,  321,  322,  323,
  324,  325,
};
#define YYFINAL 44
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 333
#if YYDEBUG
const char * const yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
"'\"'",0,0,0,"'&'",0,"'('","')'","'*'","'+'","','","'-'","'.'","'/'",0,0,0,0,0,
0,0,0,0,0,"':'","';'","'<'","'='","'>'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,"'['",0,"']'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,"'{'","'|'","'}'","'~'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"T_REGISTER","T_CONST",
"T_EXPORT","T_DOWNLOAD","T_SCB","T_SRAM","T_ALIAS","T_SIZE","T_EXPR_LSHIFT",
"T_EXPR_RSHIFT","T_ADDRESS","T_ACCESS_MODE","T_MODES","T_DEFINE",
"T_SET_SRC_MODE","T_SET_DST_MODE","T_MODE","T_BEGIN_CS","T_END_CS","T_FIELD",
"T_ENUM","T_MASK","T_NUMBER","T_PATH","T_STRING","T_ARG","T_MACROBODY",
"T_CEXPR","T_EOF","T_INCLUDE","T_VERSION","T_PREFIX","T_PATCH_ARG_LIST","T_SHR",
"T_SHL","T_ROR","T_ROL","T_MVI","T_MOV","T_CLR","T_BMOV","T_JMP","T_JC","T_JNC",
"T_JE","T_JNE","T_JNZ","T_JZ","T_CALL","T_ADD","T_ADC","T_INC","T_DEC","T_STC",
"T_CLC","T_CMP","T_NOT","T_XOR","T_TEST","T_AND","T_OR","T_RET","T_NOP",
"T_ACCUM","T_ALLONES","T_ALLZEROS","T_NONE","T_SINDEX","T_MODE_PTR","T_A",
"T_SYMBOL","T_NL","T_IF","T_ELSE","T_ELSE_IF","T_ENDIF","UMINUS",
};
const char * const yyrule[] = {
"$accept : program",
"program : include",
"program : program include",
"program : prefix",
"program : program prefix",
"program : patch_arg_list",
"program : program patch_arg_list",
"program : version",
"program : program version",
"program : register",
"program : program register",
"program : constant",
"program : program constant",
"program : macrodefn",
"program : program macrodefn",
"program : scratch_ram",
"program : program scratch_ram",
"program : scb",
"program : program scb",
"program : label",
"program : program label",
"program : set_src_mode",
"program : program set_src_mode",
"program : set_dst_mode",
"program : program set_dst_mode",
"program : critical_section_start",
"program : program critical_section_start",
"program : critical_section_end",
"program : program critical_section_end",
"program : conditional",
"program : program conditional",
"program : code",
"program : program code",
"include : T_INCLUDE '<' T_PATH '>'",
"include : T_INCLUDE '\"' T_PATH '\"'",
"prefix : T_PREFIX '=' T_STRING",
"patch_arg_list : T_PATCH_ARG_LIST '=' T_STRING",
"version : T_VERSION '=' T_STRING",
"$$1 :",
"register : T_REGISTER $$1 reg_definition",
"$$2 :",
"reg_definition : T_SYMBOL '{' $$2 reg_attribute_list '}'",
"reg_attribute_list : reg_attribute",
"reg_attribute_list : reg_attribute_list reg_attribute",
"reg_attribute : reg_address",
"reg_attribute : size",
"reg_attribute : access_mode",
"reg_attribute : modes",
"reg_attribute : field_defn",
"reg_attribute : enum_defn",
"reg_attribute : mask_defn",
"reg_attribute : alias",
"reg_attribute : accumulator",
"reg_attribute : mode_pointer",
"reg_attribute : allones",
"reg_attribute : allzeros",
"reg_attribute : none",
"reg_attribute : sindex",
"reg_address : T_ADDRESS T_NUMBER",
"size : T_SIZE T_NUMBER",
"access_mode : T_ACCESS_MODE T_MODE",
"modes : T_MODES mode_list",
"mode_list : mode_value",
"mode_list : mode_list ',' mode_value",
"mode_value : T_NUMBER",
"mode_value : T_SYMBOL",
"$$3 :",
"field_defn : T_FIELD $$3 '{' enum_entry_list '}'",
"$$4 :",
"field_defn : T_FIELD T_SYMBOL expression $$4 '{' enum_entry_list '}'",
"field_defn : T_FIELD T_SYMBOL expression",
"$$5 :",
"enum_defn : T_ENUM $$5 '{' enum_entry_list '}'",
"$$6 :",
"enum_defn : T_ENUM T_SYMBOL expression $$6 '{' enum_entry_list '}'",
"enum_entry_list : enum_entry",
"enum_entry_list : enum_entry_list ',' enum_entry",
"enum_entry : T_SYMBOL",
"enum_entry : T_SYMBOL expression",
"mask_defn : T_MASK T_SYMBOL expression",
"alias : T_ALIAS T_SYMBOL",
"accumulator : T_ACCUM",
"mode_pointer : T_MODE_PTR",
"allones : T_ALLONES",
"allzeros : T_ALLZEROS",
"none : T_NONE",
"sindex : T_SINDEX",
"expression : expression '|' expression",
"expression : expression '&' expression",
"expression : expression '+' expression",
"expression : expression '-' expression",
"expression : expression '*' expression",
"expression : expression '/' expression",
"expression : expression T_EXPR_LSHIFT expression",
"expression : expression T_EXPR_RSHIFT expression",
"expression : '(' expression ')'",
"expression : '~' expression",
"expression : '-' expression",
"expression : T_NUMBER",
"expression : T_SYMBOL",
"constant : T_CONST T_SYMBOL numerical_value",
"constant : T_CONST T_SYMBOL T_DOWNLOAD",
"macrodefn_prologue : T_DEFINE T_SYMBOL",
"macrodefn : macrodefn_prologue T_MACROBODY",
"macrodefn : macrodefn_prologue '(' macro_arglist ')' T_MACROBODY",
"macro_arglist :",
"macro_arglist : T_ARG",
"macro_arglist : macro_arglist ',' T_ARG",
"numerical_value : T_NUMBER",
"numerical_value : '-' T_NUMBER",
"$$7 :",
"$$8 :",
"$$9 :",
"scratch_ram : T_SRAM '{' $$7 reg_address $$8 size $$9 scb_or_sram_attributes '}'",
"$$10 :",
"$$11 :",
"$$12 :",
"scb : T_SCB '{' $$10 reg_address $$11 size $$12 scb_or_sram_attributes '}'",
"scb_or_sram_attributes :",
"scb_or_sram_attributes : modes",
"scb_or_sram_attributes : scb_or_sram_reg_list",
"scb_or_sram_attributes : modes scb_or_sram_reg_list",
"scb_or_sram_reg_list : reg_definition",
"scb_or_sram_reg_list : scb_or_sram_reg_list reg_definition",
"reg_symbol : T_SYMBOL",
"reg_symbol : T_SYMBOL '[' T_SYMBOL ']'",
"reg_symbol : T_SYMBOL '[' T_NUMBER ']'",
"reg_symbol : T_A",
"destination : reg_symbol",
"immediate : expression",
"immediate_or_a : expression",
"immediate_or_a : T_A",
"source : reg_symbol",
"opt_source :",
"opt_source : ',' source",
"ret :",
"ret : T_RET",
"set_src_mode : T_SET_SRC_MODE T_NUMBER ';'",
"set_dst_mode : T_SET_DST_MODE T_NUMBER ';'",
"critical_section_start : T_BEGIN_CS ';'",
"critical_section_end : T_END_CS ';'",
"export :",
"export : T_EXPORT",
"label : export T_SYMBOL ':'",
"address : T_SYMBOL",
"address : T_SYMBOL '+' T_NUMBER",
"address : T_SYMBOL '-' T_NUMBER",
"address : '.'",
"address : '.' '+' T_NUMBER",
"address : '.' '-' T_NUMBER",
"conditional : T_IF T_CEXPR '{'",
"conditional : T_ELSE T_IF T_CEXPR '{'",
"conditional : T_ELSE '{'",
"conditional : '}'",
"f1_opcode : T_AND",
"f1_opcode : T_XOR",
"f1_opcode : T_ADD",
"f1_opcode : T_ADC",
"code : f1_opcode destination ',' immediate_or_a opt_source ret ';'",
"code : T_OR reg_symbol ',' immediate_or_a opt_source ret ';'",
"code : T_INC destination opt_source ret ';'",
"code : T_DEC destination opt_source ret ';'",
"code : T_CLC ret ';'",
"code : T_CLC T_MVI destination ',' immediate_or_a ret ';'",
"code : T_STC ret ';'",
"code : T_STC destination ret ';'",
"code : T_BMOV destination ',' source ',' immediate ret ';'",
"code : T_MOV destination ',' source ret ';'",
"code : T_MVI destination ',' immediate_or_a ret ';'",
"code : T_NOT destination opt_source ret ';'",
"code : T_CLR destination ret ';'",
"code : T_NOP ret ';'",
"code : T_RET ';'",
"f2_opcode : T_SHL",
"f2_opcode : T_SHR",
"f2_opcode : T_ROL",
"f2_opcode : T_ROR",
"code : f2_opcode destination ',' expression opt_source ret ';'",
"jmp_jc_jnc_call : T_JMP",
"jmp_jc_jnc_call : T_JC",
"jmp_jc_jnc_call : T_JNC",
"jmp_jc_jnc_call : T_CALL",
"jz_jnz : T_JZ",
"jz_jnz : T_JNZ",
"je_jne : T_JE",
"je_jne : T_JNE",
"code : jmp_jc_jnc_call address ';'",
"code : T_OR reg_symbol ',' immediate jmp_jc_jnc_call address ';'",
"code : T_TEST source ',' immediate_or_a jz_jnz address ';'",
"code : T_CMP source ',' immediate_or_a je_jne address ';'",
"code : T_MOV source jmp_jc_jnc_call address ';'",
"code : T_MVI immediate jmp_jc_jnc_call address ';'",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH 10000
#endif
#endif
#define YYINITSTACKSIZE 200
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short *yyss;
short *yysslim;
YYSTYPE *yyvs;
int yystacksize;
#line 1375 "aicasm_gram.y"

static void
process_field(int field_type, symbol_t *sym, int value)
{
	/*
	 * Add the current register to its
	 * symbol list, if it already exists,
	 * warn if we are setting it to a
	 * different value, or in the bit to
	 * the "allowed bits" of this register.
	 */
	if (sym->type == UNINITIALIZED) {
		sym->type = field_type;
		initialize_symbol(sym);
		sym->info.finfo->value = value;
		if (field_type != ENUM_ENTRY) {
			if (field_type != MASK && value == 0) {
				stop("Empty Field, or Enum", EX_DATAERR);
				/* NOTREACHED */
			}
			sym->info.finfo->value = value;
			sym->info.finfo->mask = value;
		} else if (field_symbol != NULL) {
			sym->info.finfo->mask = field_symbol->info.finfo->value;
		} else {
			sym->info.finfo->mask = 0xFF;
		}
	} else if (sym->type != field_type) {
		stop("Field definition mirrors a definition of the same "
		     " name, but a different type", EX_DATAERR);
		/* NOTREACHED */
	} else if (value != sym->info.finfo->value) {
		stop("Field redefined with a conflicting value", EX_DATAERR);
		/* NOTREACHED */
	}
	/* Fail if this symbol is already listed */
	if (symlist_search(&(sym->info.finfo->symrefs),
			   cur_symbol->name) != NULL) {
		stop("Field defined multiple times for register", EX_DATAERR);
		/* NOTREACHED */
	}
	symlist_add(&(sym->info.finfo->symrefs), cur_symbol,
		    SYMLIST_INSERT_HEAD);
	cur_symbol->info.rinfo->valid_bitmask |= sym->info.finfo->mask;
	cur_symbol->info.rinfo->typecheck_masks = TRUE;
	symlist_add(&(cur_symbol->info.rinfo->fields), sym, SYMLIST_SORT);
}

static void
initialize_symbol(symbol_t *symbol)
{
	switch (symbol->type) {
	case UNINITIALIZED:
		stop("Call to initialize_symbol with type field unset",
		     EX_SOFTWARE);
		/* NOTREACHED */
		break;
	case REGISTER:
	case SRAMLOC:
	case SCBLOC:
		symbol->info.rinfo =
		    (struct reg_info *)malloc(sizeof(struct reg_info));
		if (symbol->info.rinfo == NULL) {
			stop("Can't create register info", EX_SOFTWARE);
			/* NOTREACHED */
		}
		memset(symbol->info.rinfo, 0,
		       sizeof(struct reg_info));
		SLIST_INIT(&(symbol->info.rinfo->fields));
		/*
		 * Default to allowing access in all register modes
		 * or to the mode specified by the SCB or SRAM space
		 * we are in.
		 */
		if (scb_or_sram_symbol != NULL)
			symbol->info.rinfo->modes =
			    scb_or_sram_symbol->info.rinfo->modes;
		else
			symbol->info.rinfo->modes = ~0;
		break;
	case ALIAS:
		symbol->info.ainfo =
		    (struct alias_info *)malloc(sizeof(struct alias_info));
		if (symbol->info.ainfo == NULL) {
			stop("Can't create alias info", EX_SOFTWARE);
			/* NOTREACHED */
		}
		memset(symbol->info.ainfo, 0,
		       sizeof(struct alias_info));
		break;
	case MASK:
	case FIELD:
	case ENUM:
	case ENUM_ENTRY:
		symbol->info.finfo =
		    (struct field_info *)malloc(sizeof(struct field_info));
		if (symbol->info.finfo == NULL) {
			stop("Can't create field info", EX_SOFTWARE);
			/* NOTREACHED */
		}
		memset(symbol->info.finfo, 0, sizeof(struct field_info));
		SLIST_INIT(&(symbol->info.finfo->symrefs));
		break;
	case CONST:
	case DOWNLOAD_CONST:
		symbol->info.cinfo =
		    (struct const_info *)malloc(sizeof(struct const_info));
		if (symbol->info.cinfo == NULL) {
			stop("Can't create alias info", EX_SOFTWARE);
			/* NOTREACHED */
		}
		memset(symbol->info.cinfo, 0,
		       sizeof(struct const_info));
		break;
	case LABEL:
		symbol->info.linfo =
		    (struct label_info *)malloc(sizeof(struct label_info));
		if (symbol->info.linfo == NULL) {
			stop("Can't create label info", EX_SOFTWARE);
			/* NOTREACHED */
		}
		memset(symbol->info.linfo, 0,
		       sizeof(struct label_info));
		break;
	case CONDITIONAL:
		symbol->info.condinfo =
		    (struct cond_info *)malloc(sizeof(struct cond_info));
		if (symbol->info.condinfo == NULL) {
			stop("Can't create conditional info", EX_SOFTWARE);
			/* NOTREACHED */
		}
		memset(symbol->info.condinfo, 0,
		       sizeof(struct cond_info));
		break;
	case MACRO:
		symbol->info.macroinfo = 
		    (struct macro_info *)malloc(sizeof(struct macro_info));
		if (symbol->info.macroinfo == NULL) {
			stop("Can't create macro info", EX_SOFTWARE);
			/* NOTREACHED */
		}
		memset(symbol->info.macroinfo, 0,
		       sizeof(struct macro_info));
		STAILQ_INIT(&symbol->info.macroinfo->args);
		break;
	default:
		stop("Call to initialize_symbol with invalid symbol type",
		     EX_SOFTWARE);
		/* NOTREACHED */
		break;
	}
}

static void
add_macro_arg(const char *argtext, int argnum)
{
	struct macro_arg *marg;
	int i;
	int retval;
		

	if (cur_symbol == NULL || cur_symbol->type != MACRO) {
		stop("Invalid current symbol for adding macro arg",
		     EX_SOFTWARE);
		/* NOTREACHED */
	}

	marg = (struct macro_arg *)malloc(sizeof(*marg));
	if (marg == NULL) {
		stop("Can't create macro_arg structure", EX_SOFTWARE);
		/* NOTREACHED */
	}
	marg->replacement_text = NULL;
	retval = snprintf(regex_pattern, sizeof(regex_pattern),
			  "[^-/A-Za-z0-9_](%s)([^-/A-Za-z0-9_]|$)",
			  argtext);
	if (retval >= sizeof(regex_pattern)) {
		stop("Regex text buffer too small for arg",
		     EX_SOFTWARE);
		/* NOTREACHED */
	}
	retval = regcomp(&marg->arg_regex, regex_pattern, REG_EXTENDED);
	if (retval != 0) {
		stop("Regex compilation failed", EX_SOFTWARE);
		/* NOTREACHED */
	}
	STAILQ_INSERT_TAIL(&cur_symbol->info.macroinfo->args, marg, links);
}

static void
add_macro_body(const char *bodytext)
{
	if (cur_symbol == NULL || cur_symbol->type != MACRO) {
		stop("Invalid current symbol for adding macro arg",
		     EX_SOFTWARE);
		/* NOTREACHED */
	}
	cur_symbol->info.macroinfo->body = strdup(bodytext);
	if (cur_symbol->info.macroinfo->body == NULL) {
		stop("Can't duplicate macro body text", EX_SOFTWARE);
		/* NOTREACHED */
	}
}

static void
process_register(symbol_t **p_symbol)
{
	symbol_t *symbol = *p_symbol;

	if (symbol->type == UNINITIALIZED) {
		snprintf(errbuf, sizeof(errbuf), "Undefined register %s",
			 symbol->name);
		stop(errbuf, EX_DATAERR);
		/* NOTREACHED */
	} else if (symbol->type == ALIAS) {
		*p_symbol = symbol->info.ainfo->parent;
	} else if ((symbol->type != REGISTER)
		&& (symbol->type != SCBLOC)
		&& (symbol->type != SRAMLOC)) {
		snprintf(errbuf, sizeof(errbuf),
			 "Specified symbol %s is not a register",
			 symbol->name);
		stop(errbuf, EX_DATAERR);
	}
}

static void
format_1_instr(int opcode, symbol_ref_t *dest, expression_t *immed,
	       symbol_ref_t *src, int ret)
{
	struct instruction *instr;
	struct ins_format1 *f1_instr;

	if (src->symbol == NULL)
		src = dest;

	/* Test register permissions */
	test_writable_symbol(dest->symbol);
	test_readable_symbol(src->symbol);

	/* Ensure that immediate makes sense for this destination */
	type_check(dest->symbol, immed, opcode);

	/* Allocate sequencer space for the instruction and fill it out */
	instr = seq_alloc();
	f1_instr = &instr->format.format1;
	f1_instr->ret = ret ? 1 : 0;
	f1_instr->opcode = opcode;
	f1_instr->destination = dest->symbol->info.rinfo->address
			      + dest->offset;
	f1_instr->source = src->symbol->info.rinfo->address
			 + src->offset;
	f1_instr->immediate = immed->value;

	if (is_download_const(immed))
		f1_instr->parity = 1;
	else if (dest->symbol == mode_ptr.symbol) {
		u_int src_value;
		u_int dst_value;

		/*
		 * Attempt to update mode information if
		 * we are operating on the mode register.
		 */
		if (src->symbol == allones.symbol)
			src_value = 0xFF;
		else if (src->symbol == allzeros.symbol)
			src_value = 0;
		else if (src->symbol == mode_ptr.symbol)
			src_value = (dst_mode << 4) | src_mode;
		else
			goto cant_update;

		switch (opcode) {
		case AIC_OP_AND:
			dst_value = src_value & immed->value;
			break;
		case AIC_OP_XOR:
			dst_value = src_value ^ immed->value;
			break;
		case AIC_OP_ADD:
			dst_value = (src_value + immed->value) & 0xFF;
			break;
		case AIC_OP_OR:
			dst_value = src_value | immed->value;
			break;
		case AIC_OP_BMOV:
			dst_value = src_value;
			break;
		default:
			goto cant_update;
		}
		src_mode = dst_value & 0xF;
		dst_mode = (dst_value >> 4) & 0xF;
	}

cant_update:
	symlist_free(&immed->referenced_syms);
	instruction_ptr++;
}

static void
format_2_instr(int opcode, symbol_ref_t *dest, expression_t *places,
	       symbol_ref_t *src, int ret)
{
	struct instruction *instr;
	struct ins_format2 *f2_instr;
	uint8_t shift_control;

	if (src->symbol == NULL)
		src = dest;

	/* Test register permissions */
	test_writable_symbol(dest->symbol);
	test_readable_symbol(src->symbol);

	/* Allocate sequencer space for the instruction and fill it out */
	instr = seq_alloc();
	f2_instr = &instr->format.format2;
	f2_instr->ret = ret ? 1 : 0;
	f2_instr->opcode = AIC_OP_ROL;
	f2_instr->destination = dest->symbol->info.rinfo->address
			      + dest->offset;
	f2_instr->source = src->symbol->info.rinfo->address
			 + src->offset;
	if (places->value > 8 || places->value <= 0) {
		stop("illegal shift value", EX_DATAERR);
		/* NOTREACHED */
	}
	switch (opcode) {
	case AIC_OP_SHL:
		if (places->value == 8)
			shift_control = 0xf0;
		else
			shift_control = (places->value << 4) | places->value;
		break;
	case AIC_OP_SHR:
		if (places->value == 8) {
			shift_control = 0xf8;
		} else {
			shift_control = (places->value << 4)
				      | (8 - places->value)
				      | 0x08;
		}
		break;
	case AIC_OP_ROL:
		shift_control = places->value & 0x7;
		break;
	case AIC_OP_ROR:
		shift_control = (8 - places->value) | 0x08;
		break;
	default:
		shift_control = 0; /* Quiet Compiler */
		stop("Invalid shift operation specified", EX_SOFTWARE);
		/* NOTREACHED */
		break;
	};
	f2_instr->shift_control = shift_control;
	symlist_free(&places->referenced_syms);
	instruction_ptr++;
}

static void
format_3_instr(int opcode, symbol_ref_t *src,
	       expression_t *immed, symbol_ref_t *address)
{
	struct instruction *instr;
	struct ins_format3 *f3_instr;
	int addr;

	/* Test register permissions */
	test_readable_symbol(src->symbol);

	/* Ensure that immediate makes sense for this source */
	type_check(src->symbol, immed, opcode);

	/* Allocate sequencer space for the instruction and fill it out */
	instr = seq_alloc();
	f3_instr = &instr->format.format3;
	if (address->symbol == NULL) {
		/* 'dot' referrence.  Use the current instruction pointer */
		addr = instruction_ptr + address->offset;
	} else if (address->symbol->type == UNINITIALIZED) {
		/* forward reference */
		addr = address->offset;
		instr->patch_label = address->symbol;
	} else
		addr = address->symbol->info.linfo->address + address->offset;
	f3_instr->opcode = opcode;
	f3_instr->address = addr;
	f3_instr->source = src->symbol->info.rinfo->address
			 + src->offset;
	f3_instr->immediate = immed->value;

	if (is_download_const(immed))
		f3_instr->parity = 1;

	symlist_free(&immed->referenced_syms);
	instruction_ptr++;
}

static void
test_readable_symbol(symbol_t *symbol)
{
	
	if ((symbol->info.rinfo->modes & (0x1 << src_mode)) == 0) {
		snprintf(errbuf, sizeof(errbuf),
			"Register %s unavailable in source reg mode %d",
			symbol->name, src_mode);
		stop(errbuf, EX_DATAERR);
	}

	if (symbol->info.rinfo->mode == WO) {
		stop("Write Only register specified as source",
		     EX_DATAERR);
		/* NOTREACHED */
	}
}

static void
test_writable_symbol(symbol_t *symbol)
{
	
	if ((symbol->info.rinfo->modes & (0x1 << dst_mode)) == 0) {
		snprintf(errbuf, sizeof(errbuf),
			"Register %s unavailable in destination reg mode %d",
			symbol->name, dst_mode);
		stop(errbuf, EX_DATAERR);
	}

	if (symbol->info.rinfo->mode == RO) {
		stop("Read Only register specified as destination",
		     EX_DATAERR);
		/* NOTREACHED */
	}
}

static void
type_check(symbol_t *symbol, expression_t *expression, int opcode)
{
	symbol_node_t *node;
	int and_op;

	and_op = FALSE;
	if (opcode == AIC_OP_AND || opcode == AIC_OP_JNZ || AIC_OP_JZ)
		and_op = TRUE;

	/*
	 * Make sure that we aren't attempting to write something
	 * that hasn't been defined.  If this is an and operation,
	 * this is a mask, so "undefined" bits are okay.
	 */
	if (and_op == FALSE
	 && (expression->value & ~symbol->info.rinfo->valid_bitmask) != 0) {
		snprintf(errbuf, sizeof(errbuf),
			 "Invalid bit(s) 0x%x in immediate written to %s",
			 expression->value & ~symbol->info.rinfo->valid_bitmask,
			 symbol->name);
		stop(errbuf, EX_DATAERR);
		/* NOTREACHED */
	}

	/*
	 * Now make sure that all of the symbols referenced by the
	 * expression are defined for this register.
	 */
	if (symbol->info.rinfo->typecheck_masks != FALSE) {
		for(node = expression->referenced_syms.slh_first;
		    node != NULL;
		    node = node->links.sle_next) {
			if ((node->symbol->type == MASK
			  || node->symbol->type == FIELD
			  || node->symbol->type == ENUM
			  || node->symbol->type == ENUM_ENTRY)
			 && symlist_search(&node->symbol->info.finfo->symrefs,
					   symbol->name) == NULL) {
				snprintf(errbuf, sizeof(errbuf),
					 "Invalid field or mask %s "
					 "for register %s",
					 node->symbol->name, symbol->name);
				stop(errbuf, EX_DATAERR);
				/* NOTREACHED */
			}
		}
	}
}

static void
make_expression(expression_t *immed, int value)
{
	SLIST_INIT(&immed->referenced_syms);
	immed->value = value & 0xff;
}

static void
add_conditional(symbol_t *symbol)
{
	static int numfuncs;

	if (numfuncs == 0) {
		/* add a special conditional, "0" */
		symbol_t *false_func;

		false_func = symtable_get("0");
		if (false_func->type != UNINITIALIZED) {
			stop("Conditional expression '0' "
			     "conflicts with a symbol", EX_DATAERR);
			/* NOTREACHED */
		}
		false_func->type = CONDITIONAL;
		initialize_symbol(false_func);
		false_func->info.condinfo->func_num = numfuncs++;
		symlist_add(&patch_functions, false_func, SYMLIST_INSERT_HEAD);
	}

	/* This condition has occurred before */
	if (symbol->type == CONDITIONAL)
		return;

	if (symbol->type != UNINITIALIZED) {
		stop("Conditional expression conflicts with a symbol",
		     EX_DATAERR);
		/* NOTREACHED */
	}

	symbol->type = CONDITIONAL;
	initialize_symbol(symbol);
	symbol->info.condinfo->func_num = numfuncs++;
	symlist_add(&patch_functions, symbol, SYMLIST_INSERT_HEAD);
}

static void
add_version(const char *verstring)
{
	const char prefix[] = " * ";
	int newlen;
	int oldlen;

	newlen = strlen(verstring) + strlen(prefix);
	oldlen = 0;
	if (versions != NULL)
		oldlen = strlen(versions);
	versions = realloc(versions, newlen + oldlen + 2);
	if (versions == NULL)
		stop("Can't allocate version string", EX_SOFTWARE);
	strcpy(&versions[oldlen], prefix);
	strcpy(&versions[oldlen + strlen(prefix)], verstring);
	versions[newlen + oldlen] = '\n';
	versions[newlen + oldlen + 1] = '\0';
}

void
yyerror(const char *string)
{
	stop(string, EX_DATAERR);
}

static int
is_download_const(expression_t *immed)
{
	if ((immed->referenced_syms.slh_first != NULL)
	 && (immed->referenced_syms.slh_first->symbol->type == DOWNLOAD_CONST))
		return (TRUE);

	return (FALSE);
}
#line 1395 "aicasm_gram.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack()
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;
    i = yyssp - yyss;
    newss = yyss ? (short *)realloc(yyss, newsize * sizeof *newss) :
      (short *)malloc(newsize * sizeof *newss);
    if (newss == NULL)
        return -1;
    yyss = newss;
    yyssp = newss + i;
    newvs = yyvs ? (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs) :
      (YYSTYPE *)malloc(newsize * sizeof *newvs);
    if (newvs == NULL)
        return -1;
    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab

#ifndef YYPARSE_PARAM
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG void
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif	/* ANSI-C/C++ */
#else	/* YYPARSE_PARAM */
#ifndef YYPARSE_PARAM_TYPE
#define YYPARSE_PARAM_TYPE void *
#endif
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG YYPARSE_PARAM_TYPE YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL YYPARSE_PARAM_TYPE YYPARSE_PARAM;
#endif	/* ANSI-C/C++ */
#endif	/* ! YYPARSE_PARAM */

int
yyparse (YYPARSE_PARAM_ARG)
    YYPARSE_PARAM_DECL
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate])) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#if defined(lint) || defined(__GNUC__)
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#if defined(lint) || defined(__GNUC__)
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 33:
#line 261 "aicasm_gram.y"
{
		include_file(yyvsp[-1].str, BRACKETED_INCLUDE);
	}
break;
case 34:
#line 265 "aicasm_gram.y"
{
		include_file(yyvsp[-1].str, QUOTED_INCLUDE);
	}
break;
case 35:
#line 272 "aicasm_gram.y"
{
		if (prefix != stock_prefix)
			stop("Prefix multiply defined",
			     EX_DATAERR);
		prefix = strdup(yyvsp[0].str);
		if (prefix == NULL)
			stop("Unable to record prefix", EX_SOFTWARE);
	}
break;
case 36:
#line 284 "aicasm_gram.y"
{
		if (patch_arg_list != NULL)
			stop("Patch argument list multiply defined",
			     EX_DATAERR);
		patch_arg_list = strdup(yyvsp[0].str);
		if (patch_arg_list == NULL)
			stop("Unable to record patch arg list", EX_SOFTWARE);
	}
break;
case 37:
#line 296 "aicasm_gram.y"
{ add_version(yyvsp[0].str); }
break;
case 38:
#line 300 "aicasm_gram.y"
{ cur_symtype = REGISTER; }
break;
case 40:
#line 305 "aicasm_gram.y"
{
			if (yyvsp[-1].sym->type != UNINITIALIZED) {
				stop("Register multiply defined", EX_DATAERR);
				/* NOTREACHED */
			}
			cur_symbol = yyvsp[-1].sym; 
			cur_symbol->type = cur_symtype;
			initialize_symbol(cur_symbol);
		}
break;
case 41:
#line 316 "aicasm_gram.y"
{                    
			/*
			 * Default to allowing everything in for registers
			 * with no bit or mask definitions.
			 */
			if (cur_symbol->info.rinfo->valid_bitmask == 0)
				cur_symbol->info.rinfo->valid_bitmask = 0xFF;

			if (cur_symbol->info.rinfo->size == 0)
				cur_symbol->info.rinfo->size = 1;

			/*
			 * This might be useful for registers too.
			 */
			if (cur_symbol->type != REGISTER) {
				if (cur_symbol->info.rinfo->address == 0)
					cur_symbol->info.rinfo->address =
					    sram_or_scb_offset;
				sram_or_scb_offset +=
				    cur_symbol->info.rinfo->size;
			}
			cur_symbol = NULL;
		}
break;
case 58:
#line 365 "aicasm_gram.y"
{
		cur_symbol->info.rinfo->address = yyvsp[0].value;
	}
break;
case 59:
#line 372 "aicasm_gram.y"
{
		cur_symbol->info.rinfo->size = yyvsp[0].value;
		if (scb_or_sram_symbol != NULL) {
			u_int max_addr;
			u_int sym_max_addr;

			max_addr = scb_or_sram_symbol->info.rinfo->address
				 + scb_or_sram_symbol->info.rinfo->size;
			sym_max_addr = cur_symbol->info.rinfo->address
				     + cur_symbol->info.rinfo->size;

			if (sym_max_addr > max_addr)
				stop("SCB or SRAM space exhausted", EX_DATAERR);
		}
	}
break;
case 60:
#line 391 "aicasm_gram.y"
{
		cur_symbol->info.rinfo->mode = yyvsp[0].value;
	}
break;
case 61:
#line 398 "aicasm_gram.y"
{
		cur_symbol->info.rinfo->modes = yyvsp[0].value;
	}
break;
case 62:
#line 405 "aicasm_gram.y"
{
		yyval.value = yyvsp[0].value;
	}
break;
case 63:
#line 409 "aicasm_gram.y"
{
		yyval.value = yyvsp[-2].value | yyvsp[0].value;
	}
break;
case 64:
#line 416 "aicasm_gram.y"
{
		if (yyvsp[0].value > 4) {
			stop("Valid register modes range between 0 and 4.",
			     EX_DATAERR);
			/* NOTREACHED */
		}

		yyval.value = (0x1 << yyvsp[0].value);
	}
break;
case 65:
#line 426 "aicasm_gram.y"
{
		symbol_t *symbol;

		symbol = yyvsp[0].sym;
		if (symbol->type != CONST) {
			stop("Only \"const\" symbols allowed in "
			     "mode definitions.", EX_DATAERR);
			/* NOTREACHED */
		}
		if (symbol->info.cinfo->value > 4) {
			stop("Valid register modes range between 0 and 4.",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		yyval.value = (0x1 << symbol->info.cinfo->value);
	}
break;
case 66:
#line 446 "aicasm_gram.y"
{
			field_symbol = NULL;
			enum_next_value = 0;
			enum_increment = 1;
		}
break;
case 68:
#line 453 "aicasm_gram.y"
{
			process_field(FIELD, yyvsp[-1].sym, yyvsp[0].expression.value);
			field_symbol = yyvsp[-1].sym;
			enum_next_value = 0;
			enum_increment = 0x01 << (ffs(yyvsp[0].expression.value) - 1);
		}
break;
case 70:
#line 461 "aicasm_gram.y"
{
		process_field(FIELD, yyvsp[-1].sym, yyvsp[0].expression.value);
	}
break;
case 71:
#line 468 "aicasm_gram.y"
{
			field_symbol = NULL;
			enum_next_value = 0;
			enum_increment = 1;
		}
break;
case 73:
#line 475 "aicasm_gram.y"
{
			process_field(ENUM, yyvsp[-1].sym, yyvsp[0].expression.value);
			field_symbol = yyvsp[-1].sym;
			enum_next_value = 0;
			enum_increment = 0x01 << (ffs(yyvsp[0].expression.value) - 1);
		}
break;
case 77:
#line 491 "aicasm_gram.y"
{
		process_field(ENUM_ENTRY, yyvsp[0].sym, enum_next_value);
		enum_next_value += enum_increment;
	}
break;
case 78:
#line 496 "aicasm_gram.y"
{
		process_field(ENUM_ENTRY, yyvsp[-1].sym, yyvsp[0].expression.value);
		enum_next_value = yyvsp[0].expression.value + enum_increment;
	}
break;
case 79:
#line 504 "aicasm_gram.y"
{
		process_field(MASK, yyvsp[-1].sym, yyvsp[0].expression.value);
	}
break;
case 80:
#line 511 "aicasm_gram.y"
{
		if (yyvsp[0].sym->type != UNINITIALIZED) {
			stop("Re-definition of register alias",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		yyvsp[0].sym->type = ALIAS;
		initialize_symbol(yyvsp[0].sym);
		yyvsp[0].sym->info.ainfo->parent = cur_symbol;
	}
break;
case 81:
#line 525 "aicasm_gram.y"
{
		if (accumulator.symbol != NULL) {
			stop("Only one accumulator definition allowed",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		accumulator.symbol = cur_symbol;
	}
break;
case 82:
#line 537 "aicasm_gram.y"
{
		if (mode_ptr.symbol != NULL) {
			stop("Only one mode pointer definition allowed",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		mode_ptr.symbol = cur_symbol;
	}
break;
case 83:
#line 549 "aicasm_gram.y"
{
		if (allones.symbol != NULL) {
			stop("Only one definition of allones allowed",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		allones.symbol = cur_symbol;
	}
break;
case 84:
#line 561 "aicasm_gram.y"
{
		if (allzeros.symbol != NULL) {
			stop("Only one definition of allzeros allowed",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		allzeros.symbol = cur_symbol;
	}
break;
case 85:
#line 573 "aicasm_gram.y"
{
		if (none.symbol != NULL) {
			stop("Only one definition of none allowed",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		none.symbol = cur_symbol;
	}
break;
case 86:
#line 585 "aicasm_gram.y"
{
		if (sindex.symbol != NULL) {
			stop("Only one definition of sindex allowed",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		sindex.symbol = cur_symbol;
	}
break;
case 87:
#line 597 "aicasm_gram.y"
{
		 yyval.expression.value = yyvsp[-2].expression.value | yyvsp[0].expression.value;
		 symlist_merge(&yyval.expression.referenced_syms,
			       &yyvsp[-2].expression.referenced_syms,
			       &yyvsp[0].expression.referenced_syms);
	}
break;
case 88:
#line 604 "aicasm_gram.y"
{
		yyval.expression.value = yyvsp[-2].expression.value & yyvsp[0].expression.value;
		symlist_merge(&yyval.expression.referenced_syms,
			       &yyvsp[-2].expression.referenced_syms,
			       &yyvsp[0].expression.referenced_syms);
	}
break;
case 89:
#line 611 "aicasm_gram.y"
{
		yyval.expression.value = yyvsp[-2].expression.value + yyvsp[0].expression.value;
		symlist_merge(&yyval.expression.referenced_syms,
			       &yyvsp[-2].expression.referenced_syms,
			       &yyvsp[0].expression.referenced_syms);
	}
break;
case 90:
#line 618 "aicasm_gram.y"
{
		yyval.expression.value = yyvsp[-2].expression.value - yyvsp[0].expression.value;
		symlist_merge(&(yyval.expression.referenced_syms),
			       &(yyvsp[-2].expression.referenced_syms),
			       &(yyvsp[0].expression.referenced_syms));
	}
break;
case 91:
#line 625 "aicasm_gram.y"
{
		yyval.expression.value = yyvsp[-2].expression.value * yyvsp[0].expression.value;
		symlist_merge(&(yyval.expression.referenced_syms),
			       &(yyvsp[-2].expression.referenced_syms),
			       &(yyvsp[0].expression.referenced_syms));
	}
break;
case 92:
#line 632 "aicasm_gram.y"
{
		yyval.expression.value = yyvsp[-2].expression.value / yyvsp[0].expression.value;
		symlist_merge(&(yyval.expression.referenced_syms),
			       &(yyvsp[-2].expression.referenced_syms),
			       &(yyvsp[0].expression.referenced_syms));
	}
break;
case 93:
#line 639 "aicasm_gram.y"
{
		yyval.expression.value = yyvsp[-2].expression.value << yyvsp[0].expression.value;
		symlist_merge(&yyval.expression.referenced_syms,
			       &yyvsp[-2].expression.referenced_syms,
			       &yyvsp[0].expression.referenced_syms);
	}
break;
case 94:
#line 646 "aicasm_gram.y"
{
		yyval.expression.value = yyvsp[-2].expression.value >> yyvsp[0].expression.value;
		symlist_merge(&yyval.expression.referenced_syms,
			       &yyvsp[-2].expression.referenced_syms,
			       &yyvsp[0].expression.referenced_syms);
	}
break;
case 95:
#line 653 "aicasm_gram.y"
{
		yyval.expression = yyvsp[-1].expression;
	}
break;
case 96:
#line 657 "aicasm_gram.y"
{
		yyval.expression = yyvsp[0].expression;
		yyval.expression.value = (~yyval.expression.value) & 0xFF;
	}
break;
case 97:
#line 662 "aicasm_gram.y"
{
		yyval.expression = yyvsp[0].expression;
		yyval.expression.value = -yyval.expression.value;
	}
break;
case 98:
#line 667 "aicasm_gram.y"
{
		yyval.expression.value = yyvsp[0].value;
		SLIST_INIT(&yyval.expression.referenced_syms);
	}
break;
case 99:
#line 672 "aicasm_gram.y"
{
		symbol_t *symbol;

		symbol = yyvsp[0].sym;
		switch (symbol->type) {
		case ALIAS:
			symbol = yyvsp[0].sym->info.ainfo->parent;
		case REGISTER:
		case SCBLOC:
		case SRAMLOC:
			yyval.expression.value = symbol->info.rinfo->address;
			break;
		case MASK:
		case FIELD:
		case ENUM:
		case ENUM_ENTRY:
			yyval.expression.value = symbol->info.finfo->value;
			break;
		case DOWNLOAD_CONST:
		case CONST:
			yyval.expression.value = symbol->info.cinfo->value;
			break;
		case UNINITIALIZED:
		default:
		{
			snprintf(errbuf, sizeof(errbuf),
				 "Undefined symbol %s referenced",
				 symbol->name);
			stop(errbuf, EX_DATAERR);
			/* NOTREACHED */
			break;
		}
		}
		SLIST_INIT(&yyval.expression.referenced_syms);
		symlist_add(&yyval.expression.referenced_syms, symbol, SYMLIST_INSERT_HEAD);
	}
break;
case 100:
#line 712 "aicasm_gram.y"
{
		if (yyvsp[-1].sym->type != UNINITIALIZED) {
			stop("Re-definition of symbol as a constant",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		yyvsp[-1].sym->type = CONST;
		initialize_symbol(yyvsp[-1].sym);
		yyvsp[-1].sym->info.cinfo->value = yyvsp[0].value;
	}
break;
case 101:
#line 723 "aicasm_gram.y"
{
		if (yyvsp[-2].value) {
			stop("Invalid downloaded constant declaration",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		if (yyvsp[-1].sym->type != UNINITIALIZED) {
			stop("Re-definition of symbol as a downloaded constant",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		yyvsp[-1].sym->type = DOWNLOAD_CONST;
		initialize_symbol(yyvsp[-1].sym);
		yyvsp[-1].sym->info.cinfo->value = download_constant_count++;
	}
break;
case 102:
#line 742 "aicasm_gram.y"
{
		if (yyvsp[0].sym->type != UNINITIALIZED) {
			stop("Re-definition of symbol as a macro",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		cur_symbol = yyvsp[0].sym;
		cur_symbol->type = MACRO;
		initialize_symbol(cur_symbol);
	}
break;
case 103:
#line 756 "aicasm_gram.y"
{
		add_macro_body(yyvsp[0].str);
	}
break;
case 104:
#line 760 "aicasm_gram.y"
{
		add_macro_body(yyvsp[0].str);
		cur_symbol->info.macroinfo->narg = yyvsp[-2].value;
	}
break;
case 105:
#line 767 "aicasm_gram.y"
{
		/* Macros can take no arguments */
		yyval.value = 0;
	}
break;
case 106:
#line 772 "aicasm_gram.y"
{
		yyval.value = 1;
		add_macro_arg(yyvsp[0].str, 0);
	}
break;
case 107:
#line 777 "aicasm_gram.y"
{
		if (yyvsp[-2].value == 0) {
			stop("Comma without preceeding argument in arg list",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		yyval.value = yyvsp[-2].value + 1;
		add_macro_arg(yyvsp[0].str, yyvsp[-2].value);
	}
break;
case 108:
#line 790 "aicasm_gram.y"
{
		yyval.value = yyvsp[0].value;
	}
break;
case 109:
#line 794 "aicasm_gram.y"
{
		yyval.value = -yyvsp[0].value;
	}
break;
case 110:
#line 801 "aicasm_gram.y"
{
			snprintf(errbuf, sizeof(errbuf), "%s%d", SRAM_SYMNAME,
				 num_srams);
			cur_symbol = symtable_get(SRAM_SYMNAME);
			cur_symtype = SRAMLOC;
			cur_symbol->type = SRAMLOC;
			initialize_symbol(cur_symbol);
		}
break;
case 111:
#line 810 "aicasm_gram.y"
{
			sram_or_scb_offset = cur_symbol->info.rinfo->address;
		}
break;
case 112:
#line 814 "aicasm_gram.y"
{
			scb_or_sram_symbol = cur_symbol;
		}
break;
case 113:
#line 819 "aicasm_gram.y"
{
			cur_symbol = NULL;
			scb_or_sram_symbol = NULL;
		}
break;
case 114:
#line 827 "aicasm_gram.y"
{
			cur_symbol = symtable_get(SCB_SYMNAME);
			cur_symtype = SCBLOC;
			if (cur_symbol->type != UNINITIALIZED) {
				stop("Only one SRAM definition allowed",
				     EX_SOFTWARE);
				/* NOTREACHED */
			}
			cur_symbol->type = SCBLOC;
			initialize_symbol(cur_symbol);
			/* 64 bytes of SCB space */
			cur_symbol->info.rinfo->size = 64;
		}
break;
case 115:
#line 841 "aicasm_gram.y"
{
			sram_or_scb_offset = cur_symbol->info.rinfo->address;
		}
break;
case 116:
#line 845 "aicasm_gram.y"
{
			scb_or_sram_symbol = cur_symbol;
		}
break;
case 117:
#line 850 "aicasm_gram.y"
{
			cur_symbol = NULL;
			scb_or_sram_symbol = NULL;
		}
break;
case 124:
#line 870 "aicasm_gram.y"
{
		process_register(&yyvsp[0].sym);
		yyval.sym_ref.symbol = yyvsp[0].sym;
		yyval.sym_ref.offset = 0;
	}
break;
case 125:
#line 876 "aicasm_gram.y"
{
		process_register(&yyvsp[-3].sym);
		if (yyvsp[-1].sym->type != CONST) {
			stop("register offset must be a constant", EX_DATAERR);
			/* NOTREACHED */
		}
		if ((yyvsp[-1].sym->info.cinfo->value + 1) > yyvsp[-3].sym->info.rinfo->size) {
			stop("Accessing offset beyond range of register",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		yyval.sym_ref.symbol = yyvsp[-3].sym;
		yyval.sym_ref.offset = yyvsp[-1].sym->info.cinfo->value;
	}
break;
case 126:
#line 891 "aicasm_gram.y"
{
		process_register(&yyvsp[-3].sym);
		if ((yyvsp[-1].value + 1) > yyvsp[-3].sym->info.rinfo->size) {
			stop("Accessing offset beyond range of register",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		yyval.sym_ref.symbol = yyvsp[-3].sym;
		yyval.sym_ref.offset = yyvsp[-1].value;
	}
break;
case 127:
#line 902 "aicasm_gram.y"
{
		if (accumulator.symbol == NULL) {
			stop("No accumulator has been defined", EX_DATAERR);
			/* NOTREACHED */
		}
		yyval.sym_ref.symbol = accumulator.symbol;
		yyval.sym_ref.offset = 0;
	}
break;
case 128:
#line 914 "aicasm_gram.y"
{
		test_writable_symbol(yyvsp[0].sym_ref.symbol);
		yyval.sym_ref = yyvsp[0].sym_ref;
	}
break;
case 129:
#line 922 "aicasm_gram.y"
{ yyval.expression = yyvsp[0].expression; }
break;
case 130:
#line 927 "aicasm_gram.y"
{
		if (yyvsp[0].expression.value == 0 && is_download_const(&yyvsp[0].expression) == 0) {
			snprintf(errbuf, sizeof(errbuf),
				 "\nExpression evaluates to 0 and thus "
				 "references the accumulator.\n "
				 "If this is the desired effect, use 'A' "
				 "instead.\n");
			stop(errbuf, EX_DATAERR);
		}
		yyval.expression = yyvsp[0].expression;
	}
break;
case 131:
#line 939 "aicasm_gram.y"
{
		SLIST_INIT(&yyval.expression.referenced_syms);
		yyval.expression.value = 0;
	}
break;
case 132:
#line 947 "aicasm_gram.y"
{
		test_readable_symbol(yyvsp[0].sym_ref.symbol);
		yyval.sym_ref = yyvsp[0].sym_ref;
	}
break;
case 133:
#line 954 "aicasm_gram.y"
{
		yyval.sym_ref.symbol = NULL;
		yyval.sym_ref.offset = 0;
	}
break;
case 134:
#line 959 "aicasm_gram.y"
{ yyval.sym_ref = yyvsp[0].sym_ref; }
break;
case 135:
#line 963 "aicasm_gram.y"
{ yyval.value = 0; }
break;
case 136:
#line 965 "aicasm_gram.y"
{ yyval.value = 1; }
break;
case 137:
#line 970 "aicasm_gram.y"
{
		src_mode = yyvsp[-1].value;
	}
break;
case 138:
#line 977 "aicasm_gram.y"
{
		dst_mode = yyvsp[-1].value;
	}
break;
case 139:
#line 984 "aicasm_gram.y"
{
		critical_section_t *cs;

		if (in_critical_section != FALSE) {
			stop("Critical Section within Critical Section",
			     EX_DATAERR);
			/* NOTREACHED */
		}
		cs = cs_alloc();
		cs->begin_addr = instruction_ptr;
		in_critical_section = TRUE;
	}
break;
case 140:
#line 999 "aicasm_gram.y"
{
		critical_section_t *cs;

		if (in_critical_section == FALSE) {
			stop("Unballanced 'end_cs'", EX_DATAERR);
			/* NOTREACHED */
		}
		cs = TAILQ_LAST(&cs_tailq, cs_tailq);
		cs->end_addr = instruction_ptr;
		in_critical_section = FALSE;
	}
break;
case 141:
#line 1012 "aicasm_gram.y"
{ yyval.value = 0; }
break;
case 142:
#line 1014 "aicasm_gram.y"
{ yyval.value = 1; }
break;
case 143:
#line 1019 "aicasm_gram.y"
{
		if (yyvsp[-1].sym->type != UNINITIALIZED) {
			stop("Program label multiply defined", EX_DATAERR);
			/* NOTREACHED */
		}
		yyvsp[-1].sym->type = LABEL;
		initialize_symbol(yyvsp[-1].sym);
		yyvsp[-1].sym->info.linfo->address = instruction_ptr;
		yyvsp[-1].sym->info.linfo->exported = yyvsp[-2].value;
	}
break;
case 144:
#line 1033 "aicasm_gram.y"
{
		yyval.sym_ref.symbol = yyvsp[0].sym;
		yyval.sym_ref.offset = 0;
	}
break;
case 145:
#line 1038 "aicasm_gram.y"
{
		yyval.sym_ref.symbol = yyvsp[-2].sym;
		yyval.sym_ref.offset = yyvsp[0].value;
	}
break;
case 146:
#line 1043 "aicasm_gram.y"
{
		yyval.sym_ref.symbol = yyvsp[-2].sym;
		yyval.sym_ref.offset = -yyvsp[0].value;
	}
break;
case 147:
#line 1048 "aicasm_gram.y"
{
		yyval.sym_ref.symbol = NULL;
		yyval.sym_ref.offset = 0;
	}
break;
case 148:
#line 1053 "aicasm_gram.y"
{
		yyval.sym_ref.symbol = NULL;
		yyval.sym_ref.offset = yyvsp[0].value;
	}
break;
case 149:
#line 1058 "aicasm_gram.y"
{
		yyval.sym_ref.symbol = NULL;
		yyval.sym_ref.offset = -yyvsp[0].value;
	}
break;
case 150:
#line 1066 "aicasm_gram.y"
{
		scope_t *new_scope;

		add_conditional(yyvsp[-1].sym);
		new_scope = scope_alloc();
		new_scope->type = SCOPE_IF;
		new_scope->begin_addr = instruction_ptr;
		new_scope->func_num = yyvsp[-1].sym->info.condinfo->func_num;
	}
break;
case 151:
#line 1076 "aicasm_gram.y"
{
		scope_t *new_scope;
		scope_t *scope_context;
		scope_t *last_scope;

		/*
		 * Ensure that the previous scope is either an
		 * if or and else if.
		 */
		scope_context = SLIST_FIRST(&scope_stack);
		last_scope = TAILQ_LAST(&scope_context->inner_scope,
					scope_tailq);
		if (last_scope == NULL
		 || last_scope->type == T_ELSE) {

			stop("'else if' without leading 'if'", EX_DATAERR);
			/* NOTREACHED */
		}
		add_conditional(yyvsp[-1].sym);
		new_scope = scope_alloc();
		new_scope->type = SCOPE_ELSE_IF;
		new_scope->begin_addr = instruction_ptr;
		new_scope->func_num = yyvsp[-1].sym->info.condinfo->func_num;
	}
break;
case 152:
#line 1101 "aicasm_gram.y"
{
		scope_t *new_scope;
		scope_t *scope_context;
		scope_t *last_scope;

		/*
		 * Ensure that the previous scope is either an
		 * if or and else if.
		 */
		scope_context = SLIST_FIRST(&scope_stack);
		last_scope = TAILQ_LAST(&scope_context->inner_scope,
					scope_tailq);
		if (last_scope == NULL
		 || last_scope->type == SCOPE_ELSE) {

			stop("'else' without leading 'if'", EX_DATAERR);
			/* NOTREACHED */
		}
		new_scope = scope_alloc();
		new_scope->type = SCOPE_ELSE;
		new_scope->begin_addr = instruction_ptr;
	}
break;
case 153:
#line 1127 "aicasm_gram.y"
{
		scope_t *scope_context;

		scope_context = SLIST_FIRST(&scope_stack);
		if (scope_context->type == SCOPE_ROOT) {
			stop("Unexpected '}' encountered", EX_DATAERR);
			/* NOTREACHED */
		}

		scope_context->end_addr = instruction_ptr;

		/* Pop the scope */
		SLIST_REMOVE_HEAD(&scope_stack, scope_stack_links);

		process_scope(scope_context);

		if (SLIST_FIRST(&scope_stack) == NULL) {
			stop("Unexpected '}' encountered", EX_DATAERR);
			/* NOTREACHED */
		}
	}
break;
case 154:
#line 1151 "aicasm_gram.y"
{ yyval.value = AIC_OP_AND; }
break;
case 155:
#line 1152 "aicasm_gram.y"
{ yyval.value = AIC_OP_XOR; }
break;
case 156:
#line 1153 "aicasm_gram.y"
{ yyval.value = AIC_OP_ADD; }
break;
case 157:
#line 1154 "aicasm_gram.y"
{ yyval.value = AIC_OP_ADC; }
break;
case 158:
#line 1159 "aicasm_gram.y"
{
		format_1_instr(yyvsp[-6].value, &yyvsp[-5].sym_ref, &yyvsp[-3].expression, &yyvsp[-2].sym_ref, yyvsp[-1].value);
	}
break;
case 159:
#line 1166 "aicasm_gram.y"
{
		format_1_instr(AIC_OP_OR, &yyvsp[-5].sym_ref, &yyvsp[-3].expression, &yyvsp[-2].sym_ref, yyvsp[-1].value);
	}
break;
case 160:
#line 1173 "aicasm_gram.y"
{
		expression_t immed;

		make_expression(&immed, 1);
		format_1_instr(AIC_OP_ADD, &yyvsp[-3].sym_ref, &immed, &yyvsp[-2].sym_ref, yyvsp[-1].value);
	}
break;
case 161:
#line 1183 "aicasm_gram.y"
{
		expression_t immed;

		make_expression(&immed, -1);
		format_1_instr(AIC_OP_ADD, &yyvsp[-3].sym_ref, &immed, &yyvsp[-2].sym_ref, yyvsp[-1].value);
	}
break;
case 162:
#line 1193 "aicasm_gram.y"
{
		expression_t immed;

		make_expression(&immed, -1);
		format_1_instr(AIC_OP_ADD, &none, &immed, &allzeros, yyvsp[-1].value);
	}
break;
case 163:
#line 1200 "aicasm_gram.y"
{
		format_1_instr(AIC_OP_ADD, &yyvsp[-4].sym_ref, &yyvsp[-2].expression, &allzeros, yyvsp[-1].value);
	}
break;
case 164:
#line 1207 "aicasm_gram.y"
{
		expression_t immed;

		make_expression(&immed, 1);
		format_1_instr(AIC_OP_ADD, &none, &immed, &allones, yyvsp[-1].value);
	}
break;
case 165:
#line 1214 "aicasm_gram.y"
{
		expression_t immed;

		make_expression(&immed, 1);
		format_1_instr(AIC_OP_ADD, &yyvsp[-2].sym_ref, &immed, &allones, yyvsp[-1].value);
	}
break;
case 166:
#line 1224 "aicasm_gram.y"
{
		format_1_instr(AIC_OP_BMOV, &yyvsp[-6].sym_ref, &yyvsp[-2].expression, &yyvsp[-4].sym_ref, yyvsp[-1].value);
	}
break;
case 167:
#line 1231 "aicasm_gram.y"
{
		expression_t immed;

		make_expression(&immed, 1);
		format_1_instr(AIC_OP_BMOV, &yyvsp[-4].sym_ref, &immed, &yyvsp[-2].sym_ref, yyvsp[-1].value);
	}
break;
case 168:
#line 1241 "aicasm_gram.y"
{
		format_1_instr(AIC_OP_OR, &yyvsp[-4].sym_ref, &yyvsp[-2].expression, &allzeros, yyvsp[-1].value);
	}
break;
case 169:
#line 1248 "aicasm_gram.y"
{
		expression_t immed;

		make_expression(&immed, 0xff);
		format_1_instr(AIC_OP_XOR, &yyvsp[-3].sym_ref, &immed, &yyvsp[-2].sym_ref, yyvsp[-1].value);
	}
break;
case 170:
#line 1258 "aicasm_gram.y"
{
		expression_t immed;

		make_expression(&immed, 0xff);
		format_1_instr(AIC_OP_AND, &yyvsp[-2].sym_ref, &immed, &allzeros, yyvsp[-1].value);
	}
break;
case 171:
#line 1268 "aicasm_gram.y"
{
		expression_t immed;

		make_expression(&immed, 0xff);
		format_1_instr(AIC_OP_AND, &none, &immed, &allzeros, yyvsp[-1].value);
	}
break;
case 172:
#line 1278 "aicasm_gram.y"
{
		expression_t immed;

		make_expression(&immed, 0xff);
		format_1_instr(AIC_OP_AND, &none, &immed, &allzeros, TRUE);
	}
break;
case 173:
#line 1296 "aicasm_gram.y"
{ yyval.value = AIC_OP_SHL; }
break;
case 174:
#line 1297 "aicasm_gram.y"
{ yyval.value = AIC_OP_SHR; }
break;
case 175:
#line 1298 "aicasm_gram.y"
{ yyval.value = AIC_OP_ROL; }
break;
case 176:
#line 1299 "aicasm_gram.y"
{ yyval.value = AIC_OP_ROR; }
break;
case 177:
#line 1304 "aicasm_gram.y"
{
		format_2_instr(yyvsp[-6].value, &yyvsp[-5].sym_ref, &yyvsp[-3].expression, &yyvsp[-2].sym_ref, yyvsp[-1].value);
	}
break;
case 178:
#line 1310 "aicasm_gram.y"
{ yyval.value = AIC_OP_JMP; }
break;
case 179:
#line 1311 "aicasm_gram.y"
{ yyval.value = AIC_OP_JC; }
break;
case 180:
#line 1312 "aicasm_gram.y"
{ yyval.value = AIC_OP_JNC; }
break;
case 181:
#line 1313 "aicasm_gram.y"
{ yyval.value = AIC_OP_CALL; }
break;
case 182:
#line 1317 "aicasm_gram.y"
{ yyval.value = AIC_OP_JZ; }
break;
case 183:
#line 1318 "aicasm_gram.y"
{ yyval.value = AIC_OP_JNZ; }
break;
case 184:
#line 1322 "aicasm_gram.y"
{ yyval.value = AIC_OP_JE; }
break;
case 185:
#line 1323 "aicasm_gram.y"
{ yyval.value = AIC_OP_JNE; }
break;
case 186:
#line 1328 "aicasm_gram.y"
{
		expression_t immed;

		make_expression(&immed, 0);
		format_3_instr(yyvsp[-2].value, &sindex, &immed, &yyvsp[-1].sym_ref);
	}
break;
case 187:
#line 1338 "aicasm_gram.y"
{
		format_3_instr(yyvsp[-2].value, &yyvsp[-5].sym_ref, &yyvsp[-3].expression, &yyvsp[-1].sym_ref);
	}
break;
case 188:
#line 1345 "aicasm_gram.y"
{
		format_3_instr(yyvsp[-2].value, &yyvsp[-5].sym_ref, &yyvsp[-3].expression, &yyvsp[-1].sym_ref);
	}
break;
case 189:
#line 1352 "aicasm_gram.y"
{
		format_3_instr(yyvsp[-2].value, &yyvsp[-5].sym_ref, &yyvsp[-3].expression, &yyvsp[-1].sym_ref);
	}
break;
case 190:
#line 1359 "aicasm_gram.y"
{
		expression_t immed;

		make_expression(&immed, 0);
		format_3_instr(yyvsp[-2].value, &yyvsp[-3].sym_ref, &immed, &yyvsp[-1].sym_ref);
	}
break;
case 191:
#line 1369 "aicasm_gram.y"
{
		format_3_instr(yyvsp[-2].value, &allzeros, &yyvsp[-3].expression, &yyvsp[-1].sym_ref);
	}
break;
#line 2717 "aicasm_gram.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
